{*****************************************************************************}
{    F.A.Q Delphi - www.developpez.com                                        }
{                                                                             }
{ Comment obtenir la version de Windows ?                                     }
{ Auteur : Pierre Castelain                                                   }
{                                                                             }
{ Delphi Version 2 et suprieur                                               }
{*****************************************************************************}

{*****************************************************************************}
{    Note                                                                     }
{                                                                             }
{ Il y a deux mthodes prsentes ici                                         }
{ - La premire bases sur les variables de SysUtils, est simple  mettre     }
{   en oeuvre mais donne simplement le type d'OS install                     }
{ - La deuxime utilise les APIs, et permet d'obtenir la version exacte       }
{   de L'OS y compris les services pack et le numro de construction          }
{*****************************************************************************}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

Uses Registry;

//***********************************************************************
// Premire mthode, avec les variables de SysUtils
//***********************************************************************
function WindowsVersion: string;
begin
  case Win32MajorVersion of
    3: Result:='Microsoft Windows NT 3.51';
    4: case Win32MinorVersion of
         0:  case Win32Platform of
               1: case Win32CSDVersion[1] of
                    'A':    Result:='Microsoft Windows 95';
                    'B','C':    Result:='Microsoft Windows 95 OSR2';
                  else
                    Result:='Inconnue';
                  end;
               2: Result:='Microsoft Windows NT 4.0'
             else
               Result:='Inconnue';
             end;
         10: case Win32CSDVersion[1] of
               'A':    Result:='Microsoft Windows 98';
               'B':    Result:='Microsoft Windows 98 SE';
             else
               Result:='Inconnue';
             end;
         90: Result:='Microsoft Windows Millennium Edition';
       else
         Result:='Inconnue';
       end;
    5: case Win32MinorVersion of
         0:  Result:='Microsoft Windows 2000';
         1:  Result:='Microsoft Windows XP';
         2:  Result:='Microsoft Windows Server 2003';
       else
         Result:='Inconnue';
       end;
  else
    Result:='Inconnue';
  end;
end;

//***********************************************************************
// Deuxime mthode, avec l'API pour obtenir des informations dtailles
//***********************************************************************
type
  TOSVERSIONINFOEX = record
    dwOSVersionInfoSize: DWORD;
    dwMajorVersion: DWORD;
    dwMinorVersion: DWORD;
    dwBuildNumber: DWORD;
    dwPlatformId: DWORD;
    szCSDVersion: array[0..127] of AnsiChar; { Maintenance string for PSS usage }
    wServicePackMajor: WORD;
    wServicePackMinor: WORD;
    wSuiteMask: WORD;
    wProductType: BYTE;
    wReserved: BYTE;
  end;

const
  VER_SERVER_NT                      = $80000000;
  VER_WORKSTATION_NT                 = $40000000;
  VER_SUITE_SMALLBUSINESS            = $00000001;
  VER_SUITE_ENTERPRISE               = $00000002;
  VER_SUITE_BACKOFFICE               = $00000004;
  VER_SUITE_COMMUNICATIONS           = $00000008;
  VER_SUITE_TERMINAL                 = $00000010;
  VER_SUITE_SMALLBUSINESS_RESTRICTED = $00000020;
  VER_SUITE_EMBEDDEDNT               = $00000040;
  VER_SUITE_DATACENTER               = $00000080;
  VER_SUITE_SINGLEUSERTS             = $00000100;
  VER_SUITE_PERSONAL                 = $00000200;
  VER_SUITE_BLADE                    = $00000400;

  VER_NT_WORKSTATION       = $0000001;
  VER_NT_DOMAIN_CONTROLLER = $0000002;
  VER_NT_SERVER            = $0000003;

procedure AddString(var Dest: string; const Source: string; const LineFeed: boolean = True);
begin
  if Dest = '' then
    Dest:= Source
  else
    if LineFeed then
      Dest:= Dest + #13#10 + Source
    else Dest:= Dest + Source;
end;

function SystemVersionText: string;
var
  osvi: TOSVERSIONINFOEX;
  bOsVersionInfoEx: boolean;

  ProductType : String;
begin
  result:= '';

  FillChar(osvi, SizeOf(TOSVERSIONINFOEX), 0);
  osvi.dwOSVersionInfoSize:= SizeOf(TOSVERSIONINFOEX);

  // On essaye en premier d'appeler GetVersionEx avec la structure
  // tendue. Sinon erreur on essaye avec la structure normale
  bOsVersionInfoEx:= GetVersionEx(POSVERSIONINFO(@osvi)^);
  if not bOsVersionInfoEx then
  begin
    // La structure tendue ne fonctionne pas
    osvi.dwOSVersionInfoSize:= SizeOf(TOSVERSIONINFO); 
    if not GetVersionEx(POSVERSIONINFO(@osvi)^) then 
      exit; 
  end; 

  case osvi.dwPlatformId of 
    // Tests pour la famille Winodws NT
    VER_PLATFORM_WIN32_NT: 
    begin 
      // Test du produit 
      if osvi.dwMajorVersion <= 4 then
        AddString(result, 'Microsoft Windows NT'); 
      if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion = 0) then 
        AddString(result, 'Microsoft Windows 2000');

      if bOsVersionInfoEx then  // Avec les informations tendues 
      begin 
        // Test du type de station 
        if osvi.wProductType = VER_NT_WORKSTATION then 
        begin 
          if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion = 1) then 
          AddString(result, 'Microsoft Windows XP'); 

          if (osvi.wSuiteMask and VER_SUITE_PERSONAL) <> 0 then 
            AddString(result, ' Home Edition', False) 
          else 
            AddString(result, ' Professional', False); 
        end 

        // Test du type de serveur 
        else if osvi.wProductType = VER_NT_SERVER then
        begin 
          if (osvi.dwMajorVersion = 5) and (osvi.dwMinorVersion = 1) then 
            AddString(result, 'Microsoft Windows .NET'); 

          if (osvi.wSuiteMask and VER_SUITE_DATACENTER) <> 0 then 
            AddString(result, ' DataCenter Server', False) 
          else if (osvi.wSuiteMask and VER_SUITE_ENTERPRISE) <> 0 then 
            if osvi.dwMajorVersion = 4 then 
              AddString(result, ' Advanced Server', False) 
            else 
              AddString(result, ' Enterprise Server', False)
          else if osvi.wSuiteMask = VER_SUITE_BLADE then 
            AddString(result, ' Web Server', False) 
          else
            AddString(result, ' Server', False); 
        end 
      end 
      else // Il faut utliser le registre pour les version prcdentes de NT 
      begin 
        With TRegistry.Create Do
        Try 
          RootKey := HKEY_LOCAL_MACHINE; 
          OpenKey('SYSTEM\\CurrentControlSet\\Control\\ProductOptions',False); 
          ProductType := ReadString('ProductType'); 
        Finally 
          Free; 
        End; 
        if ProductType = 'WINNT' then 
          AddString(result, ' Professional', False); 
        if ProductType = 'LANMANNT' then 
          AddString(result, ' Server', False);
        if ProductType = 'SERVERNT' then 
          AddString(result, ' Advanced Server', False); 
      end; 

     // Affichage de la version, du "service pack" ou du numro 
     // de construction si existant 

      if osvi.dwMajorVersion <= 4  then 
        AddString(result, Format('version %d.%d %s (Build %d)', 
          [osvi.dwMajorVersion, 
           osvi.dwMinorVersion,
           osvi.szCSDVersion, 
           osvi.dwBuildNumber and $FFFF])) 
      else
        AddString(result, Format('%s (Build %d)', 
          [osvi.szCSDVersion, 
           osvi.dwBuildNumber and $FFFF])); 
    end; 

     // Tests pour la famille Windows 9x 
    VER_PLATFORM_WIN32_WINDOWS: 
    begin 
      if (osvi.dwMajorVersion = 4) and (osvi.dwMinorVersion = 0) then 
      begin 
        AddString(result, 'Microsoft Windows 95'); 
        if (osvi.szCSDVersion[1] = 'C') or (osvi.szCSDVersion[1] = 'B') then 
          AddString(result, ' OSR2', False); 
      end; 

      if (osvi.dwMajorVersion = 4) and (osvi.dwMinorVersion = 10) then 
      begin
        AddString(result, 'Microsoft Windows 98'); 
        if osvi.szCSDVersion[1] = 'A' then 
          AddString(result, ' SE', False);
      end;

      if (osvi.dwMajorVersion = 4) and (osvi.dwMinorVersion = 90) then
        AddString(result, 'Microsoft Windows Millennium Edition');
    end;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  ShowMessage(WindowsVersion);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  ShowMessage(SystemVersionText);
end;

end.
